package PureMessage::Support::ChangeIdentity;

use strict;
use warnings;

use base 'Exporter';
our @EXPORT_OK = qw(get_hostname
                    set_hostname
                    get_ip_address
                    set_ip_address
                    update_pg_hba_conf);

use PureMessage::Support::ChangeIdentity::Utils qw(backup_file xlog);
use PureMessage::Support::DB::Access qw(run_db_select run_db_update);
use PureMessage::Version qw($PREFIX);
use ActiveState::Handy qw(cat);
use ActiveState::Config;
use PureMessage::Config qw(%config);
use Net::Domain qw(hostfqdn);
use Socket;

my ($hostname, $ip, $location_id);

my %host_re = (
        'etc/pmx.conf'                  =>  ['admin\s*=\s*[^@]+\@',
                                             'mail_sender\s*=\s*smtp:'],
        'etc/manager/httpd2/main.conf'  =>  'ServerName\s*',
        'etc/enduser/enduser_ui.conf'   =>  'host\s*=\s*',
        'etc/enduser/enduser.conf'      =>  'enduser_url\s*=\s*http://',
        'etc/location.conf'             =>  'hostname\s*=\s*',
        'etc/pmx-qdigest.conf'          =>  'approve_addr\s*=\s*[^@]+\@',
        );

my %ip_re = (
        'etc/pmx.conf'      =>  ['mail_sender\s*=\s*smtp:',
                                 'smtp_talk_port\s*=\s*inet\:\d+\@',
                                 '^\s*port\s*=\s*inet\:\d+\@'],
        'etc/location.conf' =>  'ip\s*=\s*',
        'etc/rpc-hosts'     =>  '',
        );

sub host_files { return keys %host_re }
sub ip_files   { return keys %ip_re }

sub get_hostname {
    return $hostname if $hostname;

    $hostname = _select_from_location_table('hostname');
    return $hostname;
}

sub get_ip_address {
    return $ip if $ip;

    $ip = _select_from_location_table('ip');
    return $ip;
}

sub set_hostname {
    my %opts = @_;
    my $old_hostname = delete $opts{from} || die "Must supply from hostname!";
    my $hostname     = delete $opts{to}   || die "Must supply to hostname!";

    # Change the DB before changing the files because the DB changing function uses 
    # the config files to determine the store names
    xlog("Changing hostname in database from '$old_hostname' to '$hostname'");
    my $table_changes = _change_hostname_in_db($old_hostname, $hostname);

    xlog("Updating files with new hostname");
    my $file_changes = _change_files(\%host_re, $old_hostname, $hostname);

    return { tables => $table_changes, files => $file_changes };
}

sub set_ip_address {
    my %opts = @_;
    my $old_ip       = delete $opts{from} || die "Must supply from IP!";
    my $ip           = delete $opts{to}   || die "Must supply to IP!";
    my $old_hostname = delete $opts{from_hostname};
    my $new_hostname = delete $opts{to_hostname};


    # Change the DB before changing the files because the DB changing function uses 
    # the config files to determine the store names
    xlog("Changing IP address in database from '$old_ip' to '$ip'"); 
    my $table_changes = _change_ip_in_db(
            from_ip => $old_ip,
            to_ip => $ip
            );
    xlog("Updating files with new IP address");
    my $file_changes = _change_files(\%ip_re, $old_ip, $ip);

    return { tables => $table_changes, files => $file_changes };
}

sub update_pg_hba_conf {
    my $ip = shift;

    my $pg_hba_conf = "$PREFIX/postgres/var/data/pg_hba.conf";
    my $backup_file = backup_file($pg_hba_conf);

    open my $FH, ">>$pg_hba_conf" 
            or die "Error opening file '$pg_hba_conf': $!";
    print $FH "\nhost\tall\tall\t$ip/32\ttrust\n";
    close $FH or die "Error closing file '$pg_hba_conf': $!";

    return { $pg_hba_conf => $backup_file };
}

sub _change_files {
    my ($re_hash, $old_val, $new_val, $suppress_warnings) = @_;

    my %changes;
    foreach my $file (keys %$re_hash) {
        my $full_path = "$PREFIX/$file";

        my $file_contents = cat($full_path);
        if (!$file_contents) {
            warn "Could not read '$full_path': $!" unless $suppress_warnings;
            next;
        }
        my $new_file_contents = _replace_after_re($file_contents, $re_hash->{$file}, $old_val, $new_val);

        if ($file_contents ne $new_file_contents) {
            xlog("    $file");
            my $backup_path = backup_file($full_path);
            open my $FH, ">$full_path" or die "Error opening file '$full_path': $!";
            print $FH $new_file_contents;
            close $FH or die "Error closing file '$full_path': $!";
            $changes{$full_path} = $backup_path;
        }
    }
    return \%changes;
}

sub _change_ip_in_db {
    my %opts = @_;
    my $old_ip       = $opts{from_ip} || die "Must supply from IP!";
    my $new_ip       = $opts{to_ip}   || die "Must supply to IP!";

    eval {
        run_db_update(store => 'pmx_resources_store',
                      sql   => "UPDATE cfg_location SET ip = '$new_ip' " .
                               "WHERE ip = '$old_ip';"
                     );
    };
    if ($@) {
        if ($@ =~ m/^Error getting dsn string/) {
            return [];
        }
        else {
            die $@;
        }
    }

    return [ 'pmx_resources_store.cfg_location.ip' ];
}

sub _change_hostname_in_db {
    my ($old_hostname, $new_hostname) = @_;

    eval {
        run_db_update(store     => 'pmx_prd_store',
                      sql       => "UPDATE prd_pmx_host SET name = '$new_hostname' " .
                                   "WHERE name = '$old_hostname';",
                     );

        run_db_update(store     => 'pmx_resources_store',
                      sql       => "UPDATE cfg_location SET hostname = '$new_hostname' " .
                                   "WHERE hostname = '$old_hostname';",
                     );
    };
    if ($@) {
        if ($@ =~ m/^Error getting dsn string/) {
            return [];
        }
        else {
            die $@;
        }
    }

    return [ 'pmx_prd_store.prd_pmx_host.name', 'pmx_resources_store.cfg_location.hostname' ];
}

sub _replace_after_re {
    my ($contents, $leading_re, $old_val, $new_val) = @_;

    if (! ref($leading_re)) {
        $leading_re = [$leading_re];
    }
    foreach my $re (@$leading_re) {
        $contents =~ s/($re)$old_val/$1$new_val/mg;
    }

    return $contents;
}

sub _get_location_id {
    return $location_id if $location_id;

    my $p = ActiveState::Config->new();
    $p->parse_file("$config{quarantine_dir}/db.conf");
    my %c = $p->as_hash();
    $location_id = $c{location_id};
    return $location_id;
}

sub _select_from_location_table {
    my $to_select = shift;
    my $loc_id = _get_location_id();

    my $rows;
    eval {
        $rows = run_db_select(store => 'pmx_quarantine_store',
                              sql   => "SELECT $to_select FROM cfg_location " .
                                       "WHERE location_id = $loc_id;");
    };
    if ($@) {
        if ($@ =~ m/^Error getting dsn string/) {
            return;
        }
        else {
            die $@;
        }
    }

    if (@$rows) {
        return $rows->[0]->[0];
    }
    return;
}

1;

