package PureMessage::Support::DB::Access;

use strict;
use warnings;

use base 'Exporter';
our @EXPORT_OK = qw(run_db_select run_db_update update_pmdb_conf);

use PureMessage::Support::ChangeIdentity::Utils qw(backup_file xlog xdie);
use PureMessage::Resources::DBH qw(DBH);
use PureMessage::Config qw(%config);
use PureMessage::Version qw($PREFIX);
use PureMessage::Database::Config;
use DBI;

sub run_db_select {
    my %opts = @_;
    my $store_id  = delete $opts{store}      || die "Must supply store ID!";
    my $sql       = delete $opts{sql}        || die "Must supply SQL!";

    my $dbh = _connect(store => $store_id);
    my $sth = $dbh->prepare($sql);
    $sth->execute() or xdie "Executing SQL statement '$sql' failed: " . $sth->errstr;
    my $ar = $sth->fetchall_arrayref;

    $sth->finish();
    $dbh->disconnect();

    return $ar;
}

sub run_db_update {
    my %opts = @_;
    my $store_id  = delete $opts{store}      || die "Must supply store ID!";
    my $sql       = delete $opts{sql}        || die "Must supply SQL!";

    my $dbh = _connect(store => $store_id);
    my $sth = $dbh->prepare($sql);
    $sth->execute() or xdie "Executing SQL statement '$sql' failed: " . $sth->errstr;

    $sth->finish();
    $dbh->disconnect();
}

sub _connect {
    my %opts = @_;
    my $store_id  = delete $opts{store} || die "Must supply store ID!";

    my $db_cfg = PureMessage::Database::Config->new();
    my $store = $db_cfg->stores()->{$store_id};
    my $dsn;
    eval { $dsn = $db_cfg->store_value($store, 'dsn') };

    if (!$dsn) {
        xdie("Error getting dsn string for store '$store_id': $@");
    }

    xlog("Connecting to database (dsn='$dsn')", 'DEBUG');
    # NOTE: We ensure that all database handles open using exactly the same
    # connection arguments so that DBI handles can be shared among multiple
    # packages.  See the following docs for more details.
    # http://perl.apache.org/docs/1.0/guide/databases.html#Opening_Connections_With_Different_Parameters
    my $connect_method = $ENV{MOD_PERL} ? "connect" : "connect_cached";
    my $dbh = DBI->$connect_method($dsn, "", "", { PrintError  => 0, 
                                                   RaiseError  => 0, 
                                                   AutoCommit  => 0,
                                                 } )
       or xdie "Failed to connect to database with DSN string '$dsn': " . $DBI::errstr;
    $dbh->{RaiseError}  = 0; 
    $dbh->{PrintError}  = 1; 
    $dbh->{AutoCommit}  = 1; 
    $dbh->{HandleError} = undef;

    return $dbh;
}

sub update_pmdb_conf {
    my %opts = @_;
    my $from_ip   = $opts{from_ip}   || '';
    my $from_host = $opts{from_host} || '';
    my $to_host   = $opts{to_host}   || '';
    my $to_ip     = $opts{to_ip}     || '';

    # Make sure we handle the case where the dsn contains a non-fqdn host.
    my $short_hostname = (split('\.', $from_host))[0] if $from_host;

    my $pmdb = "$PREFIX/etc/pmx.d/pmdb.conf";
    xlog("Updating $pmdb");
    xlog("    (host: '$from_host' => '$to_host' / ip: '$from_ip' => '$to_ip')", 'DEBUG');
    backup_file($pmdb);
    my $db_cfg = PureMessage::Database::Config->new(writable => 1);
    my $updated = 0;
    foreach my $store (values %{$db_cfg->stores()}) {
        my $update = 0;
        my $dsn;
        eval { $dsn = $db_cfg->store_value($store, 'dsn') };
        if (!$dsn) {
            xlog("    ... no dsn string found, skipping", 'DEBUG');
            next;
        }

        xlog("Examining store '$store' (dsn='$dsn')", 'DEBUG');
        # dbi:Pg:dbname=pmx_quarantine;port=5432;host=hostname.sophos.com
        if ($from_host and $to_host and $dsn =~ s/host=(?:$from_host|$short_hostname)[^;]*/host=$to_host/) {
            $update++; $updated++;
        }
        if ($from_ip and $to_ip and $dsn =~ s/host=(?:$from_ip)[^;]*/host=$to_ip/) {
            $update++; $updated++;
        }
        next if !$update;
        xlog("    ... updating with dsn of '$dsn'", 'DEBUG');
        $db_cfg->update_dsn($dsn, $store, 1);
    }
    $db_cfg->save() if $updated;
    $db_cfg->unlock();
}

1;
