package PureMessage::Support::Data;
use strict;

=head1 NAME

PureMessage::Support::Data - Information Available to Support

=head1 SYNOPSIS

 This package gives access to various information that may be useful for PureMessage
 Support in resolving any support issues that might arise.

=head1 DESCRIPTION

 Ths package has...

=over

=cut


use Exporter;
use vars qw( @ISA @EXPORT @EXPORT_OK );
use File::Spec::Functions qw(catdir);
@ISA    = qw(Exporter);
@EXPORT_OK = qw( add add_group available mime_text %SUPPORT);

my %loaded;

our %SUPPORT;
for my $path (@INC) {
    my $dir = catdir($path, split("::", __PACKAGE__));
    next unless opendir(my $d, $dir);
    while (defined(my $f = readdir($d))) {
        next unless $f =~ /^([a-z]\w+)\.pm$/;
        my $m = $1;
        my $fm = __PACKAGE__ . "::$m";
        eval "require $fm";
        warn "Error sourcing support package '$fm': $@" if $@;
        $loaded{$m} = 1;
    }
}

sub available {
    return %SUPPORT;
}

sub _textify {
    my $data = shift;
    if (ref($data) eq 'CODE') {
	return &{$data}(@_);
    }
    else {
	return $data;
    }
}

sub text {
    my ($group,$name) = @_;
    my $data;
    if ($group and $name) {
	return _textify($SUPPORT{$group}->{item}->{$name}->{text}); 
    }
    elsif($group) {
	foreach $name (keys %{$SUPPORT{$group}->{item}}) {
	    $data .= _textify($SUPPORT{$group}->{item}->{$name}->{text});
	}
    }
    else {
	foreach my $k (keys %SUPPORT) {
	    $data .= text($k);
	}
    }
    return $data;
}

sub text_display {
    my $group = shift;
    my $text;
    foreach my $name (keys %{$SUPPORT{$group}->{item}}) {
        my $attachment_name = attachment_name($group, $name);   
        $attachment_name =~ s/\.gz$//;
        $text .= "\n----- $attachment_name -----\n\n" .
                _textify($SUPPORT{$group}->{item}->{$name}->{text}, 'display');
    }
    return $text;
}

sub attachment_name {
    my ($group,$name) = @_;
    return $SUPPORT{$group}->{item}->{$name}->{name};
}

sub mime_text {
    my ($mime_sep,@support_groups) = @_;
    my $text;
    $mime_sep = $mime_sep || "___pmx_support_request" . rand(1000);
    @support_groups = keys %SUPPORT unless @support_groups;
    foreach my $group_name (@support_groups) {
	foreach my $data_name (keys %{$SUPPORT{$group_name}->{item}}) {
	    $text .= "\n--$mime_sep\n";
	    $text .= _format_mime_header(attachment_name($group_name,$data_name), text($group_name,$data_name), $SUPPORT{$group_name}{item}{$data_name}{binary});
	}
    }
    $text .= "\n--$mime_sep--\n" if $text;
    $text;
}

sub _format_mime_header {
    my ($fname,$data,$binary) = @_;
    my $text;
    $fname = $fname || "unknown.txt";
    if ($fname =~ m/.gz$/) {   
        $text .= qq|Content-Type: application/x-gzip; name="$fname"\n|;
        $text .= qq|Content-Transfer-Encoding: base64\n\n|;
        use PureMessage::MIME::Encoding qw(make_encoder);
        my $enc = make_encoder('base64');
        $text .= &$enc($data);   
        $text .= &$enc(); # get any leftover incomplete lines 
        $text .= "\n\n";
    }
    elsif ($binary) {
        $text .= qq|Content-Type: application/octet-stream; name="$fname"\n|;
        $text .= qq|Content-Transfer-Encoding: base64\n\n|;
        use PureMessage::MIME::Encoding qw(make_encoder);
        my $enc = make_encoder('base64');
        $text .= &$enc($data);   
        $text .= &$enc(); # get any leftover incomplete lines 
        $text .= "\n\n";
    }
    else {
        $text .= qq|Content-Type: text/plain; charset="iso-8859-1"; name="$fname"\n|;
        $text .= qq|Content-Transfer-Encoding: 7bit\n\n$data\n\n|;
    }
    $text;
}

sub add_group {
    my ($group,%config) = @_;
    $PureMessage::Support::Data::SUPPORT{$group}->{$_} = $config{$_} foreach keys %config;
}

sub add {
    my ($group,%a) = @_;
    $PureMessage::Support::Data::SUPPORT{$group}->{item}->{$_} = $a{$_} foreach keys %a;
}


1;
