package PureMessage::Support::Request;

use PureMessage::MessageInfo;
use PureMessage::Config qw(%config);
use PureMessage::Support;
use PureMessage::Support::Data;
use PureMessage::Support::Config qw(%support_config);
use PureMessage::_sys;
use Net::SMTP;
use Data::Dumper;

my %lic = PureMessage::_sys::lic_kv();
my $SCRIPT_SENDER         = "PureMessage Send Support Request";

# attempt to determine the support address...

# is there one in the license?
my $SUPPORT_EMAIL_ADDRESS = $lic{SupportEmail};

# no then use the one from the lang file
if (!$SUPPORT_EMAIL_ADDRESS) {
    eval {
	require PureMessage::Manager::Lang;
	$SUPPORT_EMAIL_ADDRESS = PureMessage::Manager::Lang::text('SUPPORT_EMAIL_ADDRESS');
    }
}

# then go with the default since we don't know what it is suppsed to be
$SUPPORT_EMAIL_ADDRESS = "PureMessage-support\@sophos.com" unless $SUPPORT_EMAIL_ADDRESS;


my @mail_server = @default_server = ('smtp','localhost',25); # configure with a default

# but if there is one in the pmx.conf use it instead

if ($config{mail_sender}) {
    # format is: mail_sender = smtp:mail.sophos.com:25
    @mail_server = split(/:/,$config{mail_sender});

    unless($#mail_server == 2 and $mail_server[0] =~ /smtp/) {
	warn "invalid mail sender";
	@mail_server = @default_server;
    }
}

=item new 

Create an new PureMessage::Support::Request with the default values.

=cut 

sub new {
    my $proto = shift;
    my $class = ref($proto) || $proto;
    my %self = @_;
    my $self = bless(\%self, $class);
    $self->default();
    return $self;
}

sub components {
  my $self = shift;
  if (ref($support_config{component}) eq 'ARRAY') {
    return @{$support_config{component}};
  } elsif ($support_config{component}) {
    return $support_config{component};
  } else {
    return ();
  }
}

sub attachment_groups {
  my $self = shift;
  return PureMessage::Support::Data::available();
}

sub get_attachment {
  my $self = shift;
  my $attachment = shift;
  return unless $attachment;
  return PureMessage::Support::Data::text($attachment);
}

sub get_attachment_mime {
  my $self = shift;
  my $attachment = shift;
  return unless $attachment;
  return PureMessage::Support::Data::mime_text('',$attachment);
}

sub get_attachment_for_display {
  my ($self,$attachment) = @_;
  return unless $attachment;
  return PureMessage::Support::Data::text_display($attachment);
}

sub types {
  my $self = shift;
  if (ref($support_config{type}) eq 'ARRAY') {
    return @{$support_config{type}};
  } elsif ($support_config{type}) {
    return $support_config{type};
  } else {
    return ();
  }
}

sub clear_attachments {
  my $self = shift;
  $self->{attach} = [];
}

sub add_attachment {
    my ($self,@attach) = @_;
    foreach my $a (@attach) {
        push(@{$self->{attach}},$a) unless grep {$a eq $_ } @{$self->{attach}};
    }
    return $self->{attach};
}

sub default {
    my $self = shift;
    $self->{header} = undef;
    $self->{body} = undef;
    my %available = PureMessage::Support::Data::available();
    @default_attach = keys(%available);
    $self->{attach} = \@default_attach;
    $self->{type} = $support_config{type}->[0];
    $self->{mime_sep} = "___pmx_support_request" . rand(1000);

    $self->{component} = $support_config{component}->[0];
    $self->{subject} = $support_config{default_subject}->[0];
    $self->{name} = '';
    $self->{email} = '';
    $self->{to} = $SUPPORT_EMAIL_ADDRESS;
    $self->{company} = $lic{Licensee} || '';
    (undef, $self->{mail_peerhost}, $self->{mail_peerport}) = @mail_server;
}

sub _init_header {
    my $self = shift;
    my $h = new PureMessage::MessageInfo;
    $h->value('MIME-Version' => '1.0');
    $h->value('Content-Transfer-Encoding' => '7bit');
    $h->value('Content-Type' => qq|multipart/mixed; boundary="$self->{mime_sep}";|);
    return $h;
}

sub format {
    my $self = shift;
    my $h = $self->{header} = $self->_init_header();
    my $text = '';
    $h->value('To' => $self->{to});
    $h->value('Subject' => $self->{subject} . ' | ' . $self->{name} . ' @ ' . $self->{company} . ' - ' . $self->{type});
    $h->value('From' => qq("$self->{name}" <$self->{email}>));
    $h->value('Company' => $self->{company});
    $h->value('X-Script-Sender' => $SCRIPT_SENDER);
    $h->value('X-Sophos' => 'EDN_NO_LIZA'); 

    $text = $h->as_hdr();
    $text .= "\n\n";

    # add our issue as a default group
    my $issue = $self->_format_issue();
    PureMessage::Support::Data::add_group('_default');
    PureMessage::Support::Data::add('_default',issue => { text => $issue, name => 'Support Request.txt' } );
    $text .= PureMessage::Support::Data::mime_text($self->{mime_sep},'_default',@{$self->{attach}}) . "\n";
    return $text;
}

sub _format_issue {
    my $self = shift;
    my $text .= <<EOF;

Support Request
--------------

Company   : $self->{company}
Component : $self->{component} 
Issue Type: $self->{type}

Description
-----------
$self->{description}
  
EOF

  return $text;
}

sub send {
    my $self = shift;
    my $text = $self->format();

    eval {
	my $smtp = new Net::SMTP($self->{mail_peerhost}, 
                                 Port => $self->{mail_peerport});

	die "init" unless     $smtp
			and $smtp->mail($self->{address})
			and $smtp->to($self->{to})
			and $smtp->data();

	die "send and close" unless $smtp->datasend($text) and $smtp->dataend() and $smtp->quit();

    };
    if ($@) {
	die "Unable to send Support Request\n";
    }

    return 1;
}

1;
